Attribute VB_Name = "Parameters"
'  FILE:       Parameters.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality to add, modify, delete
'              and edit parameters. Also other functions
'              specific to parameters.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

Private mcQueryParameter As New cParameter

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "Parameters."

Public Function CopyParameter(ByVal NodeIdentifier As Long, _
        Optional ByVal NodeKey As String = gstrEmptyString) _
        As Object

    ' Returns an object populated with the data for the passed
    ' in parameter identifier
    
    On Error GoTo CopyParameterErr

    Set CopyParameter = gcParameters.QueryParameter(NodeIdentifier)
            
    Exit Function
    
CopyParameterErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Set CopyParameter = Nothing
    Err.Raise vbObjectError + errCopyParameterFailed, _
            gstrSource, _
            LoadResString(errCopyParameterFailed)
    
End Function
Public Sub CreateParameter(ByVal cNewParameter As cParameter, _
        ByVal lngWorkspaceId As Long)
    
    ' Creates a new parameter record with the properties
    ' same as the those of the passed in parameter object
    
    Dim cParameterToCreate As cParameter
    
    On Error GoTo CreateParameterErr
    
    Set cParameterToCreate = cNewParameter.Clone
    cParameterToCreate.WorkspaceId = lngWorkspaceId
    
    ' Call a procedure to save the parameter record
    Call InsertParameter(cParameterToCreate)

    Set cParameterToCreate = Nothing
    
    Exit Sub
    
CreateParameterErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "CreateParameter"
    Err.Raise vbObjectError + errCreateParameterFailed, _
            gstrSource, _
            LoadResString(errCreateParameterFailed)

End Sub

Private Sub InsertParameter(ByVal cNewParameter As cParameter)
    
    Dim iAppend As Integer
    Dim sName As String
    
    On Error GoTo InsertParameterErr

    iAppend = 0
    sName = cNewParameter.ParameterName
    
    gcParameters.Add cNewParameter
    
    ' Load parameter values in the tree and list view controls
    ' Need to add an offset of gintParameter when converting between the parameter type
    ' stored in the database and the node type used by StepMaster
    Call AddNode(frmMain.treStepView, frmMain.lvwStepView, _
            cNewParameter.ParameterId, cNewParameter.ParameterName, _
            cNewParameter.WorkspaceId, gintParameter + cNewParameter.ParameterType, cNewParameter)

    ' Modify the Application Extensions sub-menu
    Call UpdateAppExtensions(cNewParameter)

    ' Set the tag for the workspace node to indicate that
    ' it has been modified.
    Call WspModified(cNewParameter.WorkspaceId)

    Exit Sub
    
InsertParameterErr:
    ' If the error raised by the add function is due to a duplication
    ' of the parameter name, then try to generate a unique name
    If Err.Number - vbObjectError = errDuplicateParameterName Then
        iAppend = iAppend + 1
        cNewParameter.ParameterName = sName & CStr(iAppend)
        ' Try to insert the parameter record again
        Resume
    End If
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInsertParameterFailed, _
            gstrSource, _
            LoadResString(errInsertParameterFailed)

End Sub
Sub ParameterOK()

    On Error GoTo ParameterOKErr

    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If frmParameters.Tag = gstrUpdateMode Then
        Call ParameterModify
    Else
        Call ParameterNew
    End If
    
    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
ParameterOKErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Error messages, have already been displayed by the
    ' called functions
    
    ' Reset the mousepointer
    Call ShowFree
    
End Sub

Sub ShowModifyParameterForm(Optional ByVal lngIdentifier As Long = 0)

    Dim lngParameterId As Long
    
    On Error GoTo ShowModifyParameterFormErr

    If lngIdentifier = 0 Then
        lngParameterId = IdentifierSelected
    Else
        lngParameterId = lngIdentifier
    End If

    BugAssert lngParameterId <> 0, "ParameterModify called when an item wasn't selected!"
    Set mcQueryParameter = gcParameters.QueryParameter(lngParameterId)
    
    frmParameters.txtParameterName = mcQueryParameter.ParameterName
    If mcQueryParameter.ParameterType = gintParameterBuiltIn Then
        frmParameters.txtParameterName.Locked = True
        Call HighlightText(frmParameters.txtParameterValue)
    Else
        Call HighlightText(frmParameters.txtParameterName)
    End If
    frmParameters.txtParameterValue = mcQueryParameter.ParameterValue
    frmParameters.txtDescription = mcQueryParameter.Description
    frmParameters.lblWorkspaceId = CStr(mcQueryParameter.WorkspaceId)
    frmParameters.lblType = CStr(mcQueryParameter.ParameterType)
    
    frmParameters.Tag = gstrUpdateMode
    
    Call ShowParametersForm(mcQueryParameter.ParameterType)
    
    Exit Sub
    
ShowModifyParameterFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errShowFormFailed, _
            mstrModuleName & "ShowModifyParameterForm", _
            LoadResString(errShowFormFailed)
    
End Sub

Private Sub ShowParametersForm(ByVal iType As ParameterType)

    ' Modify the parameter form and label captions depending on the parameter type
    Select Case iType
        Case gintParameterGeneric
            frmParameters.Caption = "Define Workspace Parameter"
            
        Case gintParameterConnect
            frmParameters.Caption = "Define Connection String"
            frmParameters.lblParameterValue.Caption = "&Connect String"
            
        Case gintParameterApplication
            frmParameters.Caption = "Define Application Extensions"
            frmParameters.lblParameterName.Caption = "&Application Title"
            frmParameters.lblParameterValue.Caption = "&Command Line"
            
    End Select
        
    ' Reset the mousepointer
    ShowFree
    
    frmParameters.Show vbModal, frmMain
    
End Sub
Sub ShowCreateParameterForm(ByVal iType As ParameterType, Optional ByVal lngWorkspaceId As Long = 0)

    Dim lngWorkspace As Long
    
    On Error GoTo ShowCreateParameterFormErr

    If iType = gintParameterBuiltIn Then
        On Error GoTo 0
        Err.Raise vbObjectError + errBuiltInUpdateOnly, _
                mstrModuleName & "ShowCreateParameterForm", LoadResString(errBuiltInUpdateOnly)
    End If
    
    If lngWorkspaceId = 0 Then
        lngWorkspace = WorkspaceSelected
    Else
        lngWorkspace = lngWorkspaceId
    End If
    
    ' Store the workspace identifier as a hidden field on the
    ' Parameter form. It will be read later by the create procedure
    frmParameters.lblWorkspaceId = Str(lngWorkspaceId)
    frmParameters.lblType = Str(iType)
    frmParameters.Tag = gstrInsertMode
    
    Call ShowParametersForm(iType)
    
    Exit Sub
    
ShowCreateParameterFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errShowFormFailed, _
            mstrModuleName & "ShowCreateParameterForm", LoadResString(errShowFormFailed)
    
End Sub

Public Sub ParameterModify()

    Dim cNewParameter As cParameter
    
    On Error GoTo ParameterModifyErr

    Set cNewParameter = mcQueryParameter.Clone
    
    If cNewParameter.ParameterType <> gintParameterBuiltIn Then
        cNewParameter.ParameterName = frmParameters.txtParameterName.Text
    End If
    cNewParameter.ParameterValue = frmParameters.txtParameterValue.Text
    cNewParameter.Description = frmParameters.txtDescription.Text
    
    ' Modify the parameter record in the array
    gcParameters.Modify cNewParameter
    
    ' Modify parameter values in the list and tree view controls
    Call UpdateView(frmMain.treStepView, frmMain.lvwStepView, _
        cNewParameter.ParameterId, _
        cNewParameter.ParameterName, _
        cNewParameter, cNewParameter.ParameterType + gintParameter)

    ' Set the tag for the workspace node to indicate that
    ' it has been modified.
    Call WspModified(cNewParameter.WorkspaceId)
    
    ' Modify the Application Extensions sub-menu
    Call UpdateAppExtensions(cNewParameter)
    
    Set cNewParameter = Nothing
    
    ' Now that we're done, unload the parameters form
    Unload frmParameters
    Exit Sub
    
ParameterModifyErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errParameterUpdateFailed
    
End Sub
Public Sub ParameterDelete( _
        Optional ByVal cParamToDelete As cParameter, _
        Optional ByVal lngIdentifier As Long = 0)

    Dim lngParameterId As Long
    
    On Error GoTo ParameterDeleteErr
    
    If cParamToDelete Is Nothing And lngIdentifier = 0 Then
        lngParameterId = IdentifierSelected
    Else
        lngParameterId = lngIdentifier
    End If
    
    ' Call a procedure to delete the parameter record
    Call DeleteParameterRecord(Parameter:=cParamToDelete, Identifier:=lngParameterId)
    
    Exit Sub
    
ParameterDeleteErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ParameterDelete"
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteParameterFailed, _
            gstrSource, _
            LoadResString(errDeleteParameterFailed)
    
End Sub

Public Sub DeleteParameterRecord( _
    Optional ByVal Parameter As cParameter, _
    Optional ByVal Identifier As Long = 0, _
    Optional ParameterKey As String = gstrEmptyString)
    ' This procedure calls all the routines necessary to delete
    ' a parameter record
    
    Dim cParamToDelete As cParameter
    Dim lngParameterId As Long
    
    On Error GoTo DeleteParameterRecordErr
    gstrSource = mstrModuleName & "DeleteParameterRecord"
    
    If Not Parameter Is Nothing Then
        Set cParamToDelete = Parameter
        lngParameterId = cParamToDelete.ParameterId
    ElseIf Identifier > 0 Then
        lngParameterId = Identifier
        Set cParamToDelete = gcParameters.QueryParameter(lngParameterId)
    ElseIf ParameterKey <> gstrEmptyString Then
        lngParameterId = MakeIdentifierValid(ParameterKey)
        Set cParamToDelete = gcParameters.QueryParameter(lngParameterId)
    Else
        On Error GoTo 0
        Err.Raise vbObjectError + errMandatoryParameterMissing, _
                gstrSource, _
                LoadResString(errMandatoryParameterMissing)
    End If
        
    gcParameters.Delete lngParameterId

    ' Delete parameter node in the list and tree view controls
    Call DeleteViewNode(frmMain.treStepView, frmMain.lvwStepView, _
        lngParameterId, cParamToDelete.ParameterType + gintParameter, cParamToDelete.WorkspaceId)
    
    ' Modify the Application Extensions sub-menu
    Call UpdateAppExtensions(cParamToDelete)
    
    ' Set the tag for the workspace node to indicate that
    ' it has been modified.
    Call WspModified(cParamToDelete.WorkspaceId)
    
    Set cParamToDelete = Nothing
    Exit Sub
    
DeleteParameterRecordErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteParameterRecordFailed, _
            gstrSource, _
            LoadResString(errDeleteParameterRecordFailed)
    
End Sub
Public Sub ParameterNew()

    Dim cNewParameter As cParameter
    
    On Error GoTo ParameterNewErr

    ' All checks for validity of the parameter values will be done
    ' by the parameter class
        
    Set cNewParameter = New cParameter
    
    ' Save parameter values to the db
    cNewParameter.ParameterName = frmParameters.txtParameterName
    cNewParameter.ParameterValue = frmParameters.txtParameterValue
    cNewParameter.Description = frmParameters.txtDescription
    cNewParameter.WorkspaceId = Val(frmParameters.lblWorkspaceId)
    cNewParameter.ParameterType = Val(frmParameters.lblType)
    
    ' Call a procedure to save the parameter record
    Call InsertParameter(cNewParameter)
    
    Set cNewParameter = Nothing
    
    ' Now that we're done, unload the parameters form
    Unload frmParameters
    
    Exit Sub
    
ParameterNewErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errParameterInsertFailed
    On Error GoTo 0
    Err.Raise vbObjectError + errParameterNewFailed, _
            gstrSource, _
            LoadResString(errParameterNewFailed)
End Sub
Public Sub CreateBuiltInParameter(lWspId As Long, strParamName As String, strParamDesc As String)

    Dim cNewParameter As cParameter
    
    On Error GoTo CreateBuiltInParameterErr

    ' All checks for validity of the parameter values will be done
    ' by the parameter class
        
    Set cNewParameter = New cParameter
    
    ' Save parameter values to the db
    cNewParameter.ParameterName = strParamName
    cNewParameter.Description = strParamDesc
    cNewParameter.WorkspaceId = lWspId
    cNewParameter.ParameterType = gintParameterBuiltIn
    
    gcParameters.Add cNewParameter
    
    ' Commit the record to the db.
    gcParameters.SaveParametersInWsp (lWspId)
    
    Set cNewParameter = Nothing
    
    Exit Sub
    
CreateBuiltInParameterErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errParameterInsertFailed
    On Error GoTo 0
    Err.Raise vbObjectError + errParameterNewFailed, _
            gstrSource, _
            LoadResString(errParameterNewFailed)
End Sub

Private Sub UpdateAppExtensions(cParameterRec As cParameter)

    On Error GoTo UpdateAppExtensionsErr
    
    If cParameterRec.ParameterType = gintParameterApplication Then
        ' Set the tag to an invalid workspace value to force the menu to be rebuild
        Call AppExtensionListDirty(cParameterRec.WorkspaceId)
        
        Call ModifyToolsMenu(cParameterRec.WorkspaceId)
    End If

    Exit Sub
    
UpdateAppExtensionsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Not a critical error - continue
    
End Sub


