VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cConnection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cConnection.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     Encapsulates the properties and methods of a connection string.
'               Contains functions to insert, update and delete
'               workspace_connections records from the database.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit
Option Base 0

' Local variable(s) to hold property value(s)
Public WorkspaceId As Long
Public ConnectionId As Long
Public ConnectionValue As String
Public Description As String
Public NodeDB As Database
Public Position As Long
Public NoCountDisplay As Boolean
Public NoExecute As Boolean
Public ParseQueryOnly As Boolean
Public QuotedIdentifiers As Boolean
Public AnsiNulls As Boolean
Public ShowQueryPlan As Boolean
Public ShowStatsTime As Boolean
Public ShowStatsIO As Boolean
Public ParseOdbcMsg As Boolean
Public RowCount As Long
Public TsqlBatchSeparator As String
Public QueryTimeOut As Long
Public ServerLanguage As String
Public CharacterTranslation As Boolean
Public RegionalSettings As Boolean

Private mstrConnectionName As String
Private mintOperation As Operation

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cConnection."

' The cSequence class is used to generate unique Connection identifiers
Private mConnectionSeq As cSequence

' The StringSM class is used to carry out string operations
Private mFieldValue As cStringSM

Private Sub AssignParameters(qyExec As DAO.QueryDef)
    ' Assigns values to the parameters in the querydef object
    ' The parameter names are cryptic to differentiate them from the field names.
    ' When the parameter names are the same as the field names, parameters in the where
    ' clause do not get created.
    
    Dim prmParam As DAO.Parameter
    
    On Error GoTo AssignParametersErr
    
    For Each prmParam In qyExec.Parameters
        Select Case prmParam.Name
            Case "[w_id]"
                prmParam.Value = WorkspaceId
            
            Case "[c_id]"
                prmParam.Value = ConnectionId
                
            Case "[c_name]"
                prmParam.Value = mstrConnectionName
                
            Case "[c_value]"
                prmParam.Value = ConnectionValue
                
            Case "[desc]"
                prmParam.Value = Description
                
            Case "[no_count]"
                prmParam.Value = NoCountDisplay
                
            Case "[no_exec]"
                prmParam.Value = NoExecute
                
            Case "[parse_only]"
                prmParam.Value = ParseQueryOnly
                
            Case "[quoted_id]"
                prmParam.Value = QuotedIdentifiers
                
            Case "[a_nulls]"
                prmParam.Value = AnsiNulls
                
            Case "[show_qp]"
                prmParam.Value = ShowQueryPlan
                
            Case "[stats_tm]"
                prmParam.Value = ShowStatsTime
                
            Case "[stats_io]"
                prmParam.Value = ShowStatsIO
                
            Case "[parse_odbc]"
                prmParam.Value = ParseOdbcMsg
                
            Case "[row_cnt]"
                prmParam.Value = RowCount
                
            Case "[batch_sep]"
                prmParam.Value = TsqlBatchSeparator
                
            Case "[qry_tmout]"
                prmParam.Value = QueryTimeOut
                
            Case "[lang]"
                prmParam.Value = ServerLanguage
                
            Case "[char_trans]"
                prmParam.Value = CharacterTranslation
                
            Case "[reg_settings]"
                prmParam.Value = RegionalSettings
                
            Case Else
                ' Write the parameter name that is faulty
                WriteError errInvalidParameter, mstrSource, prmParam.Name
                On Error GoTo 0
                Err.Raise errInvalidParameter, mstrModuleName & "AssignParameters", _
                        LoadResString(errInvalidParameter)
        End Select
    Next prmParam
    
    Exit Sub
        
AssignParametersErr:

    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errAssignParametersFailed, _
        mstrModuleName & "AssignParameters", LoadResString(errAssignParametersFailed)
    
End Sub

Public Function Clone() As cConnection

    ' Creates a copy of a given Connection
    
    Dim cCloneConn As cConnection
    
    On Error GoTo CloneErr
    
    Set cCloneConn = New cConnection
    
    ' Copy all the Connection properties to the newly
    ' created Connection
    Set cCloneConn.NodeDB = NodeDB
    cCloneConn.WorkspaceId = WorkspaceId
    cCloneConn.ConnectionId = ConnectionId
    cCloneConn.ConnectionName = mstrConnectionName
    cCloneConn.ConnectionValue = ConnectionValue
    cCloneConn.Description = Description
    cCloneConn.IndOperation = mintOperation
    cCloneConn.Position = Position
    cCloneConn.NoCountDisplay = NoCountDisplay
    cCloneConn.NoExecute = NoExecute
    cCloneConn.ParseQueryOnly = ParseQueryOnly
    cCloneConn.QuotedIdentifiers = QuotedIdentifiers
    cCloneConn.AnsiNulls = AnsiNulls
    cCloneConn.ShowQueryPlan = ShowQueryPlan
    cCloneConn.ShowStatsTime = ShowStatsTime
    cCloneConn.ShowStatsIO = ShowStatsIO
    cCloneConn.ParseOdbcMsg = ParseOdbcMsg
    cCloneConn.RowCount = RowCount
    cCloneConn.TsqlBatchSeparator = TsqlBatchSeparator
    cCloneConn.QueryTimeOut = QueryTimeOut
    cCloneConn.ServerLanguage = ServerLanguage
    cCloneConn.CharacterTranslation = CharacterTranslation
    cCloneConn.RegionalSettings = RegionalSettings
    
    ' And set the return value to the newly created Connection
    Set Clone = cCloneConn
    Set cCloneConn = Nothing
    
    Exit Function
        
CloneErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "Clone"
    On Error GoTo 0
    Err.Raise vbObjectError + errCloneFailed, mstrSource, LoadResString(errCloneFailed)

End Function
Private Sub CheckDupConnectionName()
    ' Check if the Connection name already exists in the workspace
   
    Dim rstConnection As Recordset
    Dim strSql As String
    Dim qy As DAO.QueryDef

    On Error GoTo CheckDupConnectionNameErr
    mstrSource = mstrModuleName & "CheckDupConnectionName"
    
    ' Create a recordset object to retrieve the count of all Connections
    ' for the workspace with the same name
    strSql = "Select count(*) as Connection_count " & _
        " from workspace_connections " & _
        " where workspace_id = [w_id]" & _
        " and connection_name = [c_name]" & _
        " and connection_id <> [c_id]"
    
    Set qy = NodeDB.CreateQueryDef(gstrEmptyString, strSql)
    Call AssignParameters(qy)
    
    Set rstConnection = qy.OpenRecordset(dbOpenForwardOnly)
    
    If rstConnection![Connection_count] > 0 Then
        rstConnection.Close
        qy.Close
        ShowError errDuplicateConnectionName
        On Error GoTo 0
        Err.Raise vbObjectError + errDuplicateConnectionName, _
            mstrSource, LoadResString(errDuplicateConnectionName)
    End If
    
    rstConnection.Close
    qy.Close

    Exit Sub
    
CheckDupConnectionNameErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "CheckDupConnectionName"
    On Error GoTo 0
    Err.Raise vbObjectError + errProgramError, _
        mstrSource, LoadResString(errProgramError)

End Sub
Private Sub CheckDB()
    ' Check if the database object has been initialized
   
    If NodeDB Is Nothing Then
        On Error GoTo 0
        Err.Raise vbObjectError + errInvalidDB, _
            mstrModuleName & "CheckDB", LoadResString(errInvalidDB)
    End If

End Sub
Public Property Let ConnectionName(vdata As String)

    If vdata = gstrEmptyString Then
    
        ShowError errConnectionNameMandatory
        On Error GoTo 0
        ' Propogate this error back to the caller
        Err.Raise vbObjectError + errConnectionNameMandatory, _
            mstrSource, LoadResString(errConnectionNameMandatory)
    Else
        mstrConnectionName = vdata
    End If
    
End Property

Public Property Let IndOperation(ByVal vdata As Operation)
    
    ' The valid operations are define in the cOperations
    ' class. Check if the operation is valid
    Select Case vdata
        Case QueryOp, InsertOp, UpdateOp, DeleteOp
            mintOperation = vdata
        
        Case Else
            BugAssert True
    End Select
    
End Property
Public Sub Validate()
    ' Each distinct object will have a Validate method which
    ' will check if the class properties are valid. This method
    ' will be used to check interdependant properties that
    ' cannot be validated by the let procedures.
    ' It should be called by the add and modify methods of the class
            
    ' Check if the db object is valid
    Call CheckDB
    
    ' Raise an error if the Connection name already exists in the workspace
    Call CheckDupConnectionName

End Sub
Public Sub Add()

    Dim strInsert As String
    Dim qy As DAO.QueryDef
        
    On Error GoTo AddErr
    
    ' Validate the record before trying to insert the record
    Call Validate
    
    ' Create a temporary querydef object
    strInsert = "insert into workspace_connections " & _
             "( workspace_id, connection_id, " & _
             "connection_name, connection_value, " & _
             "description, no_count_display, " & _
             "no_execute, parse_query_only, " & _
             "ANSI_quoted_identifiers, ANSI_nulls, " & _
             "show_query_plan, show_stats_time, " & _
             "show_stats_io, parse_odbc_msg_prefixes, " & _
             "row_count, tsql_batch_separator, " & _
             "query_time_out, server_language, " & _
             "character_translation, regional_settings ) " & _
            " values ( [w_id], [c_id], [c_name], [c_value], " & _
            " [desc], [no_count], [no_exec], [parse_only], " & _
            " [quoted_id], [a_nulls], [show_qp], [stats_tm], " & _
            " [stats_io], [parse_odbc], [row_cnt], [batch_sep], " & _
            " [qry_tmout], [lang], [char_trans], [reg_settings] ) "

    Set qy = NodeDB.CreateQueryDef(gstrEmptyString, strInsert)
    
    ' Call a procedure to assign the Connection values
    Call AssignParameters(qy)
    
    qy.Execute dbFailOnError
    qy.Close

    Exit Sub
        
AddErr:

    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInsertFailed, _
        mstrModuleName & "Add", LoadResString(errInsertFailed)
    
End Sub
Public Sub Delete()

    Dim strDelete As String
    Dim qy As DAO.QueryDef
        
    On Error GoTo DeleteErr
    
    ' Check if the db object is valid
    Call CheckDB
    
    strDelete = "delete from workspace_connections " & _
            " where connection_id = [c_id]"
    Set qy = NodeDB.CreateQueryDef(gstrEmptyString, strDelete)
    
    Call AssignParameters(qy)
    qy.Execute dbFailOnError
    
    qy.Close
    
    Exit Sub
        
DeleteErr:
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteFailed, _
            mstrModuleName & "Delete", LoadResString(errDeleteFailed)
    
End Sub

Public Sub Modify()

    Dim strUpdate As String
    Dim qy As QueryDef
            
    On Error GoTo ModifyErr
    
    ' Validate the updated values before trying to modify the db
    Call Validate
    
    ' Create a temporary querydef object with the modify string
    strUpdate = "update workspace_connections " & _
            " set workspace_id = [w_id], " & _
            "connection_name = [c_name], " & _
            "connection_value = [c_value], " & _
            "description = [desc], " & _
            "no_count_display = [no_count], " & _
            "no_execute = [no_exec], " & _
            "parse_query_only = [parse_only], " & _
            "ANSI_quoted_identifiers = [quoted_id], " & _
            "ANSI_nulls = [a_nulls], " & _
            "show_query_plan = [show_qp], " & _
            "show_stats_time = [stats_tm], " & _
            "show_stats_io = [stats_io], " & _
            "parse_odbc_msg_prefixes = [parse_odbc], " & _
            "row_count = [row_cnt], " & _
            "tsql_batch_separator = [batch_sep], " & _
            "query_time_out = [qry_tmout], " & _
            "server_language = [lang], " & _
            "character_translation = [char_trans], " & _
            "regional_settings = [reg_settings] " & _
            " where connection_id = [c_id]"
    Set qy = NodeDB.CreateQueryDef(gstrEmptyString, strUpdate)
    
    ' Call a procedure to assign the Connection values to the
    ' querydef object
    Call AssignParameters(qy)
    qy.Execute dbFailOnError
    
    qy.Close

    Exit Sub
        
ModifyErr:

    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errModifyFailed, _
        mstrModuleName & "Modify", LoadResString(errModifyFailed)
    
End Sub
Public Property Get ConnectionName() As String

    ConnectionName = mstrConnectionName
    
End Property

Public Property Get NextIdentifier() As Long
    
    Dim lngNextId As Long

    On Error GoTo NextIdentifierErr
    
    ' First check if the database object is valid
    Call CheckDB
    
    ' Retrieve the next identifier using the sequence class
    Set mConnectionSeq = New cSequence
    Set mConnectionSeq.IdDatabase = NodeDB
    mConnectionSeq.IdentifierColumn = "connection_id"
    lngNextId = mConnectionSeq.Identifier
    Set mConnectionSeq = Nothing
    
    NextIdentifier = lngNextId
    Exit Property
    
NextIdentifierErr:
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errIdGetFailed, _
        mstrModuleName & "NextIdentifier", LoadResString(errIdGetFailed)
    
End Property
Public Property Get IndOperation() As Operation

    IndOperation = mintOperation
    
End Property

Private Sub Class_Initialize()

    Set mFieldValue = New cStringSM
    
    ' Initialize the operation indicator variable to Query
    ' It will be modified later by the collection class when
    ' inserts, updates or deletes are performed
    mintOperation = QueryOp
    
    ' Initialize connection properties to their default values
    NoCountDisplay = DEF_NO_COUNT_DISPLAY
    NoExecute = DEF_NO_EXECUTE
    ParseQueryOnly = DEF_PARSE_QUERY_ONLY
    QuotedIdentifiers = DEF_ANSI_QUOTED_IDENTIFIERS
    AnsiNulls = DEF_ANSI_NULLS
    ShowQueryPlan = DEF_SHOW_QUERY_PLAN
    ShowStatsTime = DEF_SHOW_STATS_TIME
    ShowStatsIO = DEF_SHOW_STATS_IO
    ParseOdbcMsg = DEF_PARSE_ODBC_MSG_PREFIXES
    RowCount = DEF_ROW_COUNT
    TsqlBatchSeparator = DEF_TSQL_BATCH_SEPARATOR
    QueryTimeOut = DEF_QUERY_TIME_OUT
    ServerLanguage = DEF_SERVER_LANGUAGE
    CharacterTranslation = DEF_CHARACTER_TRANSLATION
    RegionalSettings = DEF_REGIONAL_SETTINGS
    
End Sub

Private Sub Class_Terminate()

    Set NodeDB = Nothing
    Set mFieldValue = Nothing
    
End Sub
