VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cSubStep"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cSubStep.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     This module encapsulates the properties of sub-steps
'               that are used during the execution of a workspace.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this class
Private Const mstrModuleName As String = "cSubStep"

Private mlngStepId As Long
Private mintRunning As Integer ' Number of running tasks
Private mintComplete As Integer ' Number of completed tasks
' The last iterator for this sub-step
Private mcLastIterator As cRunItDetails

Public Function NewIteration(cStepRec As cStep) As cIterator
    ' Calls a procedure to determine the next iterator value
    ' for the passed in step - returns the value to be used
    ' in the iteration.
    ' It updates the instance node with the new iteration
    ' for the step.
        
    Dim cItRec As cIterator

    On Error GoTo NewIterationErr
    
    ' Call a function that will populate an iterator record
    ' with the iterator values
    Set cItRec = NextIteration(cStepRec)

    ' Initialize the run node with the new iterator
    ' values
    If Not mcLastIterator Is Nothing Then
        If cItRec Is Nothing Then
            mcLastIterator.Value = gstrEmptyString
        Else
            mcLastIterator.Value = cItRec.Value
        
            ' And if the iterator is a list of values, then update
            ' the sequence number as well
            If mcLastIterator.IteratorType = gintValue Then
                mcLastIterator.Sequence = cItRec.SequenceNo
            End If
        End If
    End If
    
    Set NewIteration = cItRec
    Set cItRec = Nothing
    
    Exit Function
    
NewIterationErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errIterateFailed, mstrModuleName, _
        LoadResString(errIterateFailed)

End Function
Public Function NextIteration(cStepRec As cStep) As cIterator
    
    ' Retrieves the next iterator value for the passed in step -
    ' returns an iterator record with the new iterator values
        
    Dim cItRec As cIterator
    Dim vntIterators As Variant
    Dim lngValue As String

    On Error GoTo NextIterationErr
    
    vntIterators = cStepRec.Iterators
    
    If Not mcLastIterator Is Nothing Then
        ' The run node contains the iterator details
        ' Get the next value for the iterator
        If mcLastIterator.IteratorType = gintValue Then
            ' Find the next iterator that appears in the list of
            ' iterator values
            Set cItRec = NextInSequence(vntIterators, mcLastIterator.Sequence)
        Else
            lngValue = CLng(Trim$(mcLastIterator.Value))
            ' Determine whether the new iterator value falls in the
            ' range between From and To
            If (mcLastIterator.RangeStep > 0 And _
                    (mcLastIterator.RangeFrom <= mcLastIterator.RangeTo) And _
                    (mcLastIterator.RangeStep + lngValue) <= mcLastIterator.RangeTo) Or _
                    (mcLastIterator.RangeStep < 0 And _
                    (mcLastIterator.RangeFrom >= mcLastIterator.RangeTo) And _
                    (mcLastIterator.RangeStep + lngValue) >= mcLastIterator.RangeTo) Then
                Set cItRec = New cIterator
                cItRec.Value = Trim$(CStr(mcLastIterator.RangeStep + lngValue))
            Else
                Set cItRec = Nothing
            End If
        End If
    Else
        Set cItRec = Nothing
    End If
    
    Set NextIteration = cItRec
    Exit Function
    
NextIterationErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errIterateFailed, mstrModuleName, _
        LoadResString(errIterateFailed)
    
End Function
Public Sub InitializeIt(cPendingStep As cStep, _
        ColParameters As cArrParameters, _
        Optional vntIterators As Variant)
        
    ' Initializes the LastIteration structure with the iterator details for the
    ' passed in step

    On Error GoTo InitializeItErr
    
    If IsMissing(vntIterators) Then
        vntIterators = cPendingStep.Iterators
    End If
    
    If IsArray(vntIterators) And Not IsEmpty(vntIterators) Then
        mcLastIterator.IteratorName = cPendingStep.IteratorName
        If vntIterators(LBound(vntIterators)).IteratorType = _
                gintValue Then
            mcLastIterator.IteratorType = gintValue
            ' Since the sequence numbers begin at 0
            mcLastIterator.Sequence = gintMinIteratorSequence - 1
        Else
            mcLastIterator.IteratorType = gintFrom
            Call InitializeItRange(vntIterators, cPendingStep.WorkspaceId, _
                    ColParameters)
        End If
    Else
        Set mcLastIterator = Nothing
    End If
    
    Exit Sub
    
InitializeItErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errIterateFailed, mstrModuleName, _
        LoadResString(errIterateFailed)
    
End Sub

Private Sub InitializeItRange(vntIterators As Variant, ByVal lWorkspace As Long, _
        ColParameters As cArrParameters)
        
    ' Initializes the LastIteration structure for range iterators from the
    ' passed in variant containing the iterator records
    
    Dim lngIndex As Long
    Dim cItRec As cIterator
    
    On Error GoTo InitializeItRangeErr
    
    If IsArray(vntIterators) And Not IsEmpty(vntIterators) Then
        
        ' Check if the iterator range has been completely initialized
        RangeComplete (vntIterators)
        
        ' Initialize the Run node with the values for the From,
        ' To and Step boundaries
        For lngIndex = LBound(vntIterators) To UBound(vntIterators)
            Set cItRec = vntIterators(lngIndex)
            Select Case cItRec.IteratorType
                Case gintFrom
                    mcLastIterator.RangeFrom = SubstituteParameters(cItRec.Value, lWorkspace, WspParameters:=ColParameters)
                Case gintTo
                    mcLastIterator.RangeTo = SubstituteParameters(cItRec.Value, lWorkspace, WspParameters:=ColParameters)
                Case gintStep
                    mcLastIterator.RangeStep = SubstituteParameters(cItRec.Value, lWorkspace, WspParameters:=ColParameters)
                Case Else
                    On Error GoTo 0
                    Err.Raise vbObjectError + errTypeInvalid, mstrModuleName, _
                            LoadResString(errTypeInvalid)
            End Select
        Next lngIndex
        
        mcLastIterator.Value = Trim$(CStr(mcLastIterator.RangeFrom - mcLastIterator.RangeStep))
    End If
    
    Exit Sub
    
InitializeItRangeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errIterateFailed, mstrModuleName, _
        LoadResString(errIterateFailed)
    
End Sub
Private Function NextInSequence(vntIterators As Variant, _
        lngOldSequence As Long) As cIterator
        
    Dim lngIndex As Long
    Dim cItRec As cIterator

    On Error GoTo NextInSequenceErr
    
    If IsArray(vntIterators) And Not IsEmpty(vntIterators) Then
        For lngIndex = LBound(vntIterators) To UBound(vntIterators)
            Set cItRec = vntIterators(lngIndex)
            If cItRec.IteratorType <> gintValue Then
                On Error GoTo 0
                Err.Raise vbObjectError + errTypeInvalid, mstrModuleName, _
                        LoadResString(errTypeInvalid)
            End If
            If cItRec.SequenceNo = lngOldSequence + 1 Then
                Exit For
            End If
    
        Next lngIndex
        
        If cItRec.SequenceNo <> lngOldSequence + 1 Then
            Set cItRec = Nothing
        End If
    Else
        Set cItRec = Nothing
    End If
    
    Set NextInSequence = cItRec
    
    Exit Function
    
NextInSequenceErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errIterateFailed, mstrModuleName, _
        LoadResString(errIterateFailed)
    
End Function

Public Property Get LastIterator() As cRunItDetails

    Set LastIterator = mcLastIterator
    
End Property
Public Property Set LastIterator(vdata As cRunItDetails)

    Set mcLastIterator = vdata
    
End Property

Public Property Get TasksRunning() As Integer

    TasksRunning = mintRunning
    
End Property

Public Property Let TasksRunning(ByVal vdata As Integer)
    
    mintRunning = vdata

End Property

Public Property Get TasksComplete() As Integer

    TasksComplete = mintComplete
    
End Property
Public Property Let TasksComplete(ByVal vdata As Integer)
    
    mintComplete = vdata

End Property
Public Property Get StepId() As Long

    StepId = mlngStepId
    
End Property
Public Property Let StepId(ByVal vdata As Long)
    
    mlngStepId = vdata

End Property
