Attribute VB_Name = "Startup"
'  FILE:       Startup.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module contains startup and cleanup functions for the project.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

Public Const LISTVIEW_BUTTON = 14

Public gstrProjectPath As String

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "Startup."

Private Sub Initialize()

    On Error GoTo InitializeErr

    ReDim gsContCriteria(gintOnFailureAbort To gintOnFailureAsk) As String
    gsContCriteria(gintOnFailureAbort) = "Abort"
    gsContCriteria(gintOnFailureContinue) = "Continue"
    gsContCriteria(gintOnFailureCompleteSiblings) = "Execute sibling steps and stop"
    gsContCriteria(gintOnFailureAbortSiblings) = "Abort sibling steps and execute next parent"
    gsContCriteria(gintOnFailureSkipSiblings) = "Skip sibling steps and execute next parent"
    gsContCriteria(gintOnFailureAsk) = "Ask"
    
    ReDim gsExecutionStatus(gintDisabled To gintAborted) As String
    gsExecutionStatus(gintDisabled) = "Disabled"
    gsExecutionStatus(gintPending) = "Pending"
    gsExecutionStatus(gintRunning) = "Running"
    gsExecutionStatus(gintComplete) = "Complete"
    gsExecutionStatus(gintFailed) = "Failed"
    gsExecutionStatus(gintAborted) = "Stopped"
    
#If Not RUN_ONLY Then
    ' Call a procedure to change the style of the toolbar
    ' on the Step Properties form
    Call SetTBar97(frmSteps.tblConstraintCommands)
    
#End If
    
    Call InitRunEngine
    
    Exit Sub
    
InitializeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "Initialize"
    Call ShowError(errInitializeFailed)
    
End Sub
Sub Main()
    
    On Error GoTo MainErr

    ' Mousepointer should indicate busy
    Call ShowBusy
    
    ' Display the Splash screen while we carry out some initialization
    frmSplash.Show
    frmSplash.Refresh

    gstrProjectPath = App.Path
    
    ' Open the database
    If OpenDBFile() = False Then
        Unload frmSplash
        Exit Sub
    End If
    
#If Not RUN_ONLY Then
    Load frmMain
    
    ' Enable the Stop Run menu options only when a workspace is
    ' actually running
    Call EnableStop(False)
    
    ' Clear all application extension menu items
    Call ClearToolsMenu
#End If
    
    Call Initialize
    
    ' Mousepointer - ready to accept user input
    Call ShowFree
    
    ' Unload the Splash screen and display the main form
    Unload frmSplash

#If RUN_ONLY Then
    frmWorkspaceOpen.Caption = gsCaptionRunWsp
    
    Call ShowWorkspacesInDb(dbsAttTool)
#Else
    frmMain.Show
#End If
    
    Exit Sub
    
MainErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Call ShowFree
    Call ShowError(errMainFailed)
    
End Sub
Private Function OpenDBFile() As Boolean
    Dim sDb As String

    On Error GoTo OpenDBFileErr
    
#If RUN_ONLY Then
    ' Always use the registry setting for the run_only mode
    sDb = DefaultDBFile()
#Else
    ' Check if the user has specified the workspace defn. file to open on the command line
    ' Else, use the registry setting
    sDb = IIf(StringEmpty(Command), DefaultDBFile(), Command)
        
    If Len(sDb) > 0 Then
        ' Trim off the enclosing double-quotes if any
        If Mid(sDb, 1, 1) = gstrDQ Then
            If Len(sDb) > 1 Then
                sDb = Mid(sDb, 2)
            Else
                sDb = gstrEmptyString
            End If
        End If
    End If

    If Len(sDb) > 0 Then
        If Mid(sDb, Len(sDb), 1) = gstrDQ Then
            If Len(sDb) > 1 Then
                sDb = Mid(sDb, 1, Len(sDb) - 1)
            Else
                sDb = gstrEmptyString
            End If
        End If
    End If
#End If
    
    ' Open the database
    OpenDBFile = SMOpenDatabase(sDb)
    
    Exit Function
    
OpenDBFileErr:
    Call LogErrors(Errors)
    OpenDBFile = False
    
End Function
Public Sub Cleanup()
    
    On Error GoTo CleanupErr

    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
#If Not RUN_ONLY Then
    ' Close all open workspaces - will also prompt for unsaved
    ' changes
    Call CloseOpenWorkspaces
#End If
    
    ' Close all open files
    Call CloseOpenFiles
    
    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
CleanupErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Resume Next
    
End Sub
